@@ -5,14 +5,36 @@
 
 LANG=C
 SCRIPT_DIR=$(dirname $(readlink -f "$0"))
-TOP=$(readlink -f "$SCRIPT_DIR/../")
+if [ -z "$TOP" ]; then
+	TOP=$(readlink -f "$SCRIPT_DIR/../")
+	if [ -f "$TOP/configure.ac" ]; then
+		# inside git
+		TEST_TOP="$TOP/tests/"
+		INTERNAL_BIN="$TOP"
+	else
+		# external, defaults to system binaries
+		TOP=$(dirname `which btrfs`)
+		TEST_TOP="$SCRIPT_DIR"
+		INTERNAL_BIN="$TEST_TOP"
+	fi
+else
+	# assume external, TOP set from commandline
+	TEST_TOP="$SCRIPT_DIR"
+	INTERNAL_BIN="$TEST_TOP"
+fi
+if ! [ -x "$TOP/btrfs" ]; then
+	echo "ERROR: cannot execute btrfs from TOP=$TOP"
+	exit 1
+fi
 TEST_DEV=${TEST_DEV:-}
-RESULTS="$TOP/tests/convert-tests-results.txt"
-IMAGE="$TOP/tests/test.img"
+RESULTS="$TEST_TOP/convert-tests-results.txt"
+IMAGE="$TEST_TOP/test.img"
 
-source "$TOP/tests/common"
-source "$TOP/tests/common.convert"
+source "$TEST_TOP/common"
+source "$TEST_TOP/common.convert"
 
+export INTERNAL_BIN
+export TEST_TOP
 export TOP
 export RESULTS
 export LANG
@@ -54,7 +76,7 @@ run_one_test() {
 }
 
 # Test special images
-for i in $(find "$TOP/tests/convert-tests" -maxdepth 1 -mindepth 1 -type d \
+for i in $(find "$TEST_TOP/convert-tests" -maxdepth 1 -mindepth 1 -type d \
 	   ${TEST:+-name "$TEST"} | sort)
 do
 	run_one_test "$i"
